<?php

namespace Drupal\one_time_password\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Messenger\MessengerTrait;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\user\Entity\User;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Event subscriber used to force enabling OTP if the setting is set.
 */
class ForceOneTimePasswordSubscriber implements EventSubscriberInterface {

  use StringTranslationTrait;
  use MessengerTrait;

  /**
   * Routes to ignore for OTP check.
   */
  protected const IGNORE_ROUTES = [
    'one_time_password.setup_form',
    'one_time_password.settings',
    'user.logout',
    'system.css_asset',
    'system.js_asset',
  ];

  public function __construct(
    protected AccountInterface $currentUser,
    protected RouteMatchInterface $routeMatch,
    protected ConfigFactoryInterface $configFactory,
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      KernelEvents::REQUEST => 'onKernelRequest',
    ];
  }

  /**
   * Redirect to OTP setup page if the user doesn't have OTP enabled.
   *
   * @param \Symfony\Component\HttpKernel\Event\RequestEvent $event
   *   The kernel event object.
   */
  public function onKernelRequest(RequestEvent $event): void {
    $enforce_opt_enabled = $this->configFactory->get('one_time_password.settings')->get('force_otp');

    // If enforcing isn't enabled or user is anonymous exit.
    if (!$enforce_opt_enabled || !$this->currentUser->isAuthenticated()) {
      return;
    }

    $uid = $this->currentUser->id();
    $user = User::load($uid);

    // Send redirect response to the OTP setup form.
    if (!in_array($this->routeMatch->getRouteName(), static::IGNORE_ROUTES, TRUE) && $user->one_time_password->isEmpty()) {
      $this->messenger()->addWarning($this->t('You must enable two factor authentication to continue.'));
      $response = new RedirectResponse(Url::fromRoute('one_time_password.setup_form', ['user' => $uid])->toString());
      $event->setResponse($response);
    }
  }

}
